@@ -25,11 +25,15 @@ $VERSION = '0.02' ;
   
   *Safe_World_NOW = \$Safe::World::NOW ;
 
-###########
-# HEADERS #
-###########
+#########
+# DUMMY #
+#########
 
 sub headers {}
+sub stdout_data {}
+sub buffer_data {}
+
+sub flush {}
 
 #########
 # PRINT #
@@ -72,12 +76,6 @@ sub close_headers {
   return 1 ;
 }
 
-#########
-# FLUSH #
-#########
-
-sub flush {}
-
 #############
 # TIEHANDLE #
 #############
